#!/bin/bash

# Copyright 2009, Nokia Corporation
#
# Mika Korhonen <ext-mika.2.korhonen@nokia.com>
# 11/2009


print_usage()
{
    echo "Usage: $0 -w <build-dir> {-k <kernel-image> -m <modules-dir> | -K} [-o <output-image>] [-z]"
}

APE_ALGO_SRC=/usr/share/ape-algo/src
APE_ALGO_PATH=/usr/share/ape-algo/bin
APE_ALGO_SRC_K=$APE_ALGO_SRC/kernel

while getopts "w:k:m:o:zK" opt; do
    case $opt in
	w)
	    WORK_DIR=$OPTARG
	    ;;
	k)
	    KERNEL_ZIMAGE=$OPTARG
	    ;;
	m)
	    KERNEL_MOD_DIR=$OPTARG
	    ;;
	o)
	    APE_ALGO_FILE=$OPTARG
	    ;;
	z)
	    ROOTFS_COMPRESS=lzop
	    ;;
	K)
	    KERNEL_ZIMAGE=$APE_ALGO_SRC_K/zImage
	    KERNEL_MOD_DIR=$(ls -1d $APE_ALGO_SRC_K/lib/modules/* | tail -1)
	    ;;
	\?)
	    print_usage
	    exit 1
	    ;;
    esac
done


[ -z "$WORK_DIR" ] && {
    print_usage
    exit 1
}

[ -d "$WORK_DIR" ] || {
    echo Working directory must exist
    exit 1
}


PATH=$PATH:$APE_ALGO_PATH
ROOT_DIR=$WORK_DIR/rootfs
ROOT_ARCHIVE=$WORK_DIR/rootfs.cpio

KERNEL_MODS_G="g_softupd g_nokia"
KERNEL_MODS="mmc_block hsi_char ssi_protocol cmt twl4030_wdt omap_ssi omap_hsmmc omap_hs vfat cryptodev rtc-twl4030"
KERNEL_MOD_DEP=$KERNEL_MOD_DIR/modules.dep

UTIL_LIST=$APE_ALGO_SRC/util-list
SKEL_LIST=$APE_ALGO_SRC/skeleton-list

SOFTUPD=softupd

[ -f "$KERNEL_ZIMAGE" ] || {
    echo Cannot find kernel image "$KERNEL_ZIMAGE"
    exit 1
}

[ -d "$KERNEL_MOD_DIR" ] || {
    echo Cannot find kernel modules directory "$KERNEL_MOD_DIR"
    exit 1
}

if [ -h "$KERNEL_MOD_DIR" ] ; then
    KERNEL_MOD_DIR_NAME=$(basename $(readlink $KERNEL_MOD_DIR))
    echo Modules directory is a symbolic link, real path is $KERNEL_MOD_DIR_NAME
else
    KERNEL_MOD_DIR_NAME=$(basename $KERNEL_MOD_DIR)
fi


[ -z "$APE_ALGO_FILE" ] && {
    APE_ALGO_FILE=$WORK_DIR/ape-algo.bin
}

rm -rf $ROOT_DIR $WORK_DIR/rootfs.*

mkdir -p $ROOT_DIR

# busybox
rm -f /tmp/ape-algo-build-*
TMPFILE=$(mktemp /tmp/ape-algo-build-XXXXXX) || exit 1
dpkg -L busybox | grep "^/.*bin/" > $TMPFILE || exit 1
dpkg -L busybox-symlinks-net-tools | grep "^/.*bin/" >> $TMPFILE || exit 1
(tar cf - -T $TMPFILE | tar xfp - -C $ROOT_DIR) || exit 1
rm -f $TMPFILE

# secbins
TMPFILE=$(mktemp /tmp/ape-algo-build-XXXXXX) || exit 1
dpkg -L libbb5-secbins | grep "^/lib/firmware/.*\.bin" > $TMPFILE || exit 1
(tar cf - -T $TMPFILE | tar xfp - -C $ROOT_DIR) || exit 1
rm -f $TMPFILE


mkdir -p $ROOT_DIR/etc
touch $ROOT_DIR/etc/mtab
install -m 644 $APE_ALGO_SRC/ape-algo-release $ROOT_DIR/etc

mkdir -p $ROOT_DIR/usr/sbin
install -m 755 /usr/sbin/softupd $ROOT_DIR/usr/sbin || exit 1

install -m 755 $APE_ALGO_SRC/init $ROOT_DIR/init

TMPFILE=$(mktemp /tmp/ape-algo-build-XXXXXX) || exit 1
ldd /usr/sbin/softupd | sed -n 's/.*=> //p' | sed 's/ (.*)$//' > $TMPFILE.ldd
cat $UTIL_LIST $TMPFILE.ldd | sort | uniq > $TMPFILE
rm -f $TMPFILE.ldd
tar chf - -T $TMPFILE | tar xfp - -C $ROOT_DIR
rm -f $TMPFILE

TARGET_KERNEL_MOD_DIR=$ROOT_DIR/lib/modules/$KERNEL_MOD_DIR_NAME
mkdir -p $TARGET_KERNEL_MOD_DIR
cp -p $KERNEL_MOD_DEP $TARGET_KERNEL_MOD_DIR/

for i in $KERNEL_MODS_G ; do
    [ -f $KERNEL_MOD_DIR/$i.ko ] && {
	KERNEL_MODS="$KERNEL_MODS $i"
	break
    }
done

KERNEL_MOD_BASE=${KERNEL_MOD_DIR%/lib/modules/*}

for i in $KERNEL_MODS ; do \
    AUX_MODS=$(sed -n "s/.*$i.ko: //p" < $KERNEL_MOD_DEP)
    [ -z "$AUX_MODS" ] || {
	for j in $AUX_MODS; do \
	    cp -p $KERNEL_MOD_BASE/$j $TARGET_KERNEL_MOD_DIR/
	done
    }
    [ -f $KERNEL_MOD_DIR/$i.ko ] && {
	cp -p $KERNEL_MOD_DIR/$i.ko $TARGET_KERNEL_MOD_DIR/
    }
done

$APE_ALGO_PATH/gen_initramfs_list.sh -o $ROOT_ARCHIVE \
    -u squash -g squash $SKEL_LIST $ROOT_DIR

[ -z "$ROOTFS_COMPRESS" ] || {
    lzop --best $ROOT_ARCHIVE && {
	ROOT_ARCHIVE=$ROOT_ARCHIVE.lzo
	echo Compressed initramfs archive using LZO
    }
}

cat $KERNEL_ZIMAGE $ROOT_ARCHIVE > $APE_ALGO_FILE
echo APE algorithm is ready at $APE_ALGO_FILE
